@@ -22,6 +22,7 @@
 
 #include "config.h"
 #include <sys/wait.h>
+#include <sys/stat.h>
 #include <unistd.h>
 #include <time.h>
 #include <stdio.h>
@@ -42,6 +43,10 @@
 #include "seahorse-gconf.h"
 #include "seahorse-vfs-data.h"
 
+#ifdef WITH_SSH
+#include "seahorse-ssh-key.h"
+#endif
+
 static const gchar *bad_filename_chars = "/\\<>|";
 
 void
@@ -210,83 +215,39 @@ seahorse_util_set_text_view_string (GtkTextView *view, GString *string)
 	gtk_text_buffer_set_text (buffer, string->str, string->len);
 }
 
-/**
- * seahorse_util_write_data_to_file:
- * @path: Path of file to write to
- * @data: Data to write
- * @release: Whether to release @data or not
- *
- * Tries to write @data to the file @path. 
- *
- * Returns: GPG_ERR_NO_ERROR if write is successful,
- * GPGME_File_Error if the file could not be opened
- **/
-gpgme_error_t
-seahorse_util_write_data_to_file (const gchar *path, gpgme_data_t data, 
-                                  gboolean release)
-{
-    gpgme_error_t err = GPG_OK;
-	gpgme_data_t file;
-	gchar *buffer;
-	gint nread;
-   
-    gpgme_data_seek (data, 0, SEEK_SET);
-	
-    file = seahorse_vfs_data_create (path, SEAHORSE_VFS_WRITE, &err);
-    if (file != NULL) {
-    	buffer = g_new0 (gchar, 128);
-    	
-    	while ((nread = gpgme_data_read (data, buffer, 128)) > 0) {
-            if(gpgme_data_write (file, buffer, nread) < 0) {
-                gpg_err_code_t e = gpg_err_code_from_errno (errno);
-                err = GPG_E (e);                
-                break;
-            }
-        }
-
-        g_free (buffer);
-    }
-
-    if (release)
-        gpgme_data_release (data);
-    
-    gpgme_data_release (file);	
-	return err;
-}
-
 /**
  * seahorse_util_write_data_to_text:
  * @data: Data to write
- * @release: Whether to release @data or not
+ * @len: Length of the data
  *
  * Converts @data to a string. 
  *
  * Returns: The string read from data
  **/
 gchar*
-seahorse_util_write_data_to_text (gpgme_data_t data, gboolean release)
+seahorse_util_write_data_to_text (gpgme_data_t data, guint *len)
 {
-	gint size = 128;
+    gint size = 128;
     gchar *buffer, *text;
     guint nread = 0;
-	GString *string;
+    GString *string;
 
     gpgme_data_seek (data, 0, SEEK_SET);
 
-	string = g_string_new ("");
-	buffer = g_new (gchar, size);
-	
-	while ((nread = gpgme_data_read (data, buffer, size)) > 0)
-		string = g_string_append_len (string, buffer, nread);
-
-    if (release)
-        gpgme_data_release (data);
+    string = g_string_new ("");
+    buffer = g_new (gchar, size);
     
-	text = string->str;
-	g_string_free (string, FALSE);
+    while ((nread = gpgme_data_read (data, buffer, size)) > 0)
+        string = g_string_append_len (string, buffer, nread);
+
+    if (len)
+        *len = string->len;
+    
+    text = string->str;
+    g_string_free (string, FALSE);
     g_free (buffer);
-	
-	return text;
+    
+    return text;
 }
 
 /** 
@@ -362,6 +323,8 @@ seahorse_util_print_fd (int fd, const char* s)
         r = write (fd, s, l);
         
         if (r == -1) {
+            if (errno == EPIPE)
+                return FALSE;
             if (errno != EAGAIN && errno != EINTR) {
                 g_critical ("couldn't write data to socket: %s", strerror (errno));
                 return FALSE;
@@ -412,11 +375,8 @@ seahorse_util_filename_for_keys (GList *keys)
         g_return_val_if_fail (SEAHORSE_IS_KEY (keys->data), NULL);
         skey = SEAHORSE_KEY (keys->data);
         t = seahorse_key_get_simple_name (skey);
-        if (t == NULL) {
-            t = g_strdup (seahorse_key_get_keyid (skey));
-            if (strlen (t) > 8)
-                t[8] = 0;
-        }
+        if (t == NULL)
+            t = g_strdup (_("Key Data"));
     } else {
         t = g_strdup (_("Multiple Keys"));
     }
@@ -752,6 +712,63 @@ seahorse_util_uris_package (const gchar* package, const char** uris)
     return TRUE;
 }
 
+GQuark
+seahorse_util_detect_mime_type (const gchar *mime)
+{
+    if (!mime || g_ascii_strcasecmp (mime, GNOME_VFS_MIME_TYPE_UNKNOWN) == 0) {
+        g_warning ("couldn't get mime type for data");
+        return 0;
+    }
+
+    if (g_ascii_strcasecmp (mime, "application/pgp-keys") == 0)
+        return SKEY_PGP;
+    
+#ifdef WITH_SSH 
+    else if (g_ascii_strcasecmp (mime, "application/x-ssh-key") == 0)
+        return SKEY_SSH;
+#endif 
+    
+    g_warning ("unsupported type of key data: %s", mime);
+    return 0;
+}
+
+GQuark
+seahorse_util_detect_data_type (const gchar *data, guint length)
+{
+    const gchar* mime = gnome_vfs_get_mime_type_for_data (data, length);
+    return seahorse_util_detect_mime_type (mime);
+}
+
+GQuark
+seahorse_util_detect_file_type (const gchar *uri)
+{
+    GnomeVFSResult res;
+    GnomeVFSFileInfo *info;
+    GQuark ret = 0;
+    
+    info = gnome_vfs_file_info_new ();
+    res = gnome_vfs_get_file_info (uri, info, 
+            GNOME_VFS_FILE_INFO_GET_MIME_TYPE | GNOME_VFS_FILE_INFO_FORCE_SLOW_MIME_TYPE);
+    
+    if (res == GNOME_VFS_OK)
+        ret = seahorse_util_detect_mime_type (info->mime_type);
+    else
+        g_warning ("couldn't get mime type for: %s: %s", uri, 
+                   gnome_vfs_result_to_string (res));
+
+    gnome_vfs_file_info_unref (info);
+    return ret;
+}
+
+gboolean
+seahorse_util_write_file_private (const gchar* filename, const gchar* contents, GError **err)
+{
+    mode_t mask = umask (0077);
+    gboolean ret = g_file_set_contents (filename, contents, -1, err);
+    umask (mask);
+    return ret;
+}
+
 GtkWidget*
 seahorse_util_chooser_save_new (const gchar *title, GtkWindow *parent)
 {
@@ -919,7 +936,8 @@ gboolean
 seahorse_util_check_suffix (const gchar *path, SeahorseSuffix suffix)
 {
 	if (suffix == SEAHORSE_SIG_SUFFIX)
-        return g_str_has_suffix (path, SEAHORSE_EXT_SIG);
+        return g_str_has_suffix (path, SEAHORSE_EXT_SIG) || 
+               g_str_has_suffix (path, SEAHORSE_EXT_ASC);
 	else
         return g_str_has_suffix (path, SEAHORSE_EXT_PGP) ||
                g_str_has_suffix (path, SEAHORSE_EXT_GPG) ||
@@ -947,15 +965,13 @@ seahorse_util_add_suffix (const gchar *path, SeahorseSuffix suffix, const gchar
     const gchar *ext;
     gchar *uri;
     gchar *t;
-	
-	if (suffix == SEAHORSE_CRYPT_SUFFIX)
-        ext = seahorse_gconf_get_boolean (ARMOR_KEY) ? SEAHORSE_EXT_ASC : SEAHORSE_EXT_PGP;
-	else if (suffix == SEAHORSE_ASC_SUFFIX)
-		ext = SEAHORSE_EXT_ASC;
-	else
-		ext = SEAHORSE_EXT_SIG;
-	
-	uri = g_strdup_printf ("%s%s", path, ext);
+    
+    if (suffix == SEAHORSE_CRYPT_SUFFIX)
+        ext = SEAHORSE_EXT_PGP;
+    else
+        ext = SEAHORSE_EXT_SIG;
+    
+    uri = g_strdup_printf ("%s%s", path, ext);
     
     if (prompt && uri && seahorse_util_uri_exists (uri)) {
             
@@ -964,7 +980,7 @@ seahorse_util_add_suffix (const gchar *path, SeahorseSuffix suffix, const gchar
         g_free (t);
 
         seahorse_util_chooser_show_key_files (dialog);
-		gtk_file_chooser_select_uri (GTK_FILE_CHOOSER (dialog), uri);
+        gtk_file_chooser_select_uri (GTK_FILE_CHOOSER (dialog), uri);
 
         g_free (uri);                
         uri = NULL;
@@ -990,8 +1006,9 @@ seahorse_util_remove_suffix (const gchar *path, const gchar *prompt)
     GtkWidget *dialog;
     gchar *uri;
     gchar *t;
-
-	uri =  g_strndup (path, strlen (path) - 4);
+    
+    g_return_val_if_fail (path != NULL, NULL);
+    uri =  g_strndup (path, strlen (path) - 4);
    
     if (prompt && uri && seahorse_util_uri_exists (uri)) {
             
@@ -1031,6 +1048,15 @@ seahorse_util_strvec_dup (const gchar** vec)
     return ret;
 }
 
+guint 
+seahorse_util_strvec_length (const gchar **vec)
+{
+    guint len = 0;
+    while (*(vec++))
+        len++;
+    return len;
+}
+
 /* TODO: This function no longer makes sense once we 
    have multiple key and encryption types */
 gpgme_key_t* 
@@ -1193,6 +1219,36 @@ seahorse_util_string_slist_equal (GSList *l1, GSList *l2)
     return !l1 && !l2;   
 }
 
+/* Callback to determine where a popup menu should be placed */
+void
+seahorse_util_determine_popup_menu_position (GtkMenu *menu, int *x, int *y,
+                                             gboolean *push_in, gpointer  gdata)
+{
+        GtkWidget      *widget;
+        GtkRequisition  requisition;
+        gint            menu_xpos;
+        gint            menu_ypos;
+
+        widget = GTK_WIDGET (gdata);
+
+        gtk_widget_size_request (GTK_WIDGET (menu), &requisition);
+
+        gdk_window_get_origin (widget->window, &menu_xpos, &menu_ypos);
+
+        menu_xpos += widget->allocation.x;
+        menu_ypos += widget->allocation.y;
+
+
+        if (menu_ypos > gdk_screen_get_height (gtk_widget_get_screen (widget)) / 2)
+                menu_ypos -= requisition.height;
+        else
+                menu_ypos += widget->allocation.height;
+           
+        *x = menu_xpos;
+        *y = menu_ypos;
+        *push_in = TRUE;
+}            
+
 /* -----------------------------------------------------------------------------
  * DNS-SD Stuff 
  */
