@@ -73,7 +73,7 @@ void        seahorse_util_handle_error          (GError*            err,
                                                  const char*        desc, ...);
 
 gchar*      seahorse_util_write_data_to_text    (gpgme_data_t       data,
-                                                 gboolean           release);
+                                                 guint              *len);
 
 guint       seahorse_util_read_data_block       (GString            *buf, 
                                                  gpgme_data_t       data, 
@@ -144,6 +144,8 @@ gchar*      seahorse_util_remove_suffix     (const gchar        *path,
 
 gchar**     seahorse_util_strvec_dup        (const gchar        **vec);
 
+guint       seahorse_util_strvec_length       (const gchar      **vec);
+
 gpgme_key_t* seahorse_util_keylist_to_keys    (GList *keys);
 
 GList*       seahorse_util_keylist_sort       (GList *keys);
